@@ -142,22 +142,6 @@ ews_populate_string_list (EContact *contact,
 	g_list_free (l_values);
 }
 
-static void
-ews_populate_cert (EContact *contact,
-                    EContactField field,
-                    gpointer value,
-                    gpointer user_data)
-{
-	GSList *list = value;
-	GBytes *bytes = list->data;
-	EContactCert cert;
-
-	cert.data = (gpointer) g_bytes_get_data (bytes, &cert.length);
-	cert.length = g_bytes_get_size (bytes);
-
-	e_contact_set (contact, E_CONTACT_X509_CERT, &cert);
-}
-
 static void
 ews_populate_photo (EContact *contact,
                     EContactField field,
@@ -231,10 +215,7 @@ static const struct prop_field_mapping {
 	{EWS_PT_PAGER_NUMBER, E_CONTACT_PHONE_PAGER, ews_populate_simple_string},
 	{EWS_PT_COMMENT, E_CONTACT_NOTE, ews_populate_simple_string},
 	{EWS_PT_DL_MEMBERS, E_CONTACT_EMAIL, ews_populate_string_list},
-	{EWS_PT_THUMBNAIL_PHOTO, E_CONTACT_PHOTO, ews_populate_photo},
-	{EWS_PT_OFFICE_LOCATION, E_CONTACT_OFFICE, ews_populate_simple_string},
-	{EWS_PT_X509_CERT, E_CONTACT_X509_CERT, ews_populate_cert},
-	{EWS_PT_SEND_RICH_INFO, E_CONTACT_WANTS_HTML, ews_populate_simple_string},
+	{EWS_PT_THUMBNAIL_PHOTO, E_CONTACT_PHOTO, ews_populate_photo}
 };
 
 static void
@@ -402,16 +383,14 @@ ews_oab_read_upto (GInputStream *is,
 	GString *str;
 
 	str = g_string_sized_new (size);
-	while (1) {
+	do {
 		gsize len;
 		gsize bytes_read;
 		gchar *c = g_malloc0 (size);
 
-		if (!g_input_stream_read_all (is, c, size, &bytes_read,
-					      cancellable, error)) {
-			g_free (c);
+		g_input_stream_read_all (is, c, size, &bytes_read, cancellable, error);
+		if (*error)
 			break;
-		}
 
 		if (bytes_read != size)
 			size = bytes_read;
@@ -420,8 +399,6 @@ ews_oab_read_upto (GInputStream *is,
 		if (len)
 			str = g_string_append_len (str, c, len);
 
-		g_free (c);
-
 		if (len == 0 || len < size) {
 			goffset seek = (goffset) len + 1 - (goffset) size;
 
@@ -431,7 +408,8 @@ ews_oab_read_upto (GInputStream *is,
 		}
 
 		size *= 2;
-	}
+		g_free (c);
+	} while (!*error);
 
 	return g_string_free (str, FALSE);
 }
@@ -615,31 +593,35 @@ ews_decode_uint32 (EwsOabDecoder *eod,
 	return ret;
 }
 
-static GBytes *
+static gchar *
 ews_decode_binary (EwsOabDecoder *eod,
                    GCancellable *cancellable,
                    GError **error)
 {
 	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 	guint32 len;
-	gchar *binary;
-	GBytes *val = NULL;
+	gchar *binary, *filename = NULL;
+	gint fd = 0;
 
 	len = ews_decode_uint32 (eod, cancellable, error);
 	if (*error)
 		return NULL;
+
 	binary = g_malloc (len);
 	g_input_stream_read (G_INPUT_STREAM (priv->fis), binary, len, cancellable, error);
-	if (*error) {
-		g_free (binary);
+	if (*error)
 		goto exit;
-	}
 
-	val = g_bytes_new_take (binary, len);
-	binary = NULL;
+	filename = g_build_filename (priv->cache_dir, "XXXXXX", NULL);
+	fd = g_mkstemp (filename);
+	g_file_set_contents (filename, binary, len, error);
 
 exit:
-	return val;
+	if (binary)
+		g_free (binary);
+	close (fd);
+
+	return filename;
 }
 
 static gpointer
@@ -690,7 +672,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 		case EWS_PTYP_BINARY:
 		{
 			ret_val = ews_decode_binary (eod, cancellable, error);
-			d (g_print ("prop id %X prop type: binary size %zd \n", prop_id, g_bytes_get_size ((GBytes *)ret_val)));
+			d (g_print ("prop id %X prop type: binary value %s \n", prop_id, (gchar *) ret_val);)
 			break;
 		}
 		case EWS_PTYP_MULTIPLEINTEGER32:
@@ -721,35 +703,23 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 						g_slist_free (list);
 						return NULL;
 					}
-				} else if (prop_type == EWS_PTYP_MULTIPLEBINARY) {
-					GBytes *val;
-
-					val = ews_decode_binary (eod, cancellable, error);
-					if (!val) {
-						g_slist_foreach (list, (GFunc) g_bytes_unref, NULL);
-						g_slist_free (list);
-						return NULL;
-					}
-
-					d (g_print ("prop id %X prop type: multi-bin size %zd\n", prop_id, g_bytes_get_size (val)));
-
-					list = g_slist_prepend (list, val);
 				} else {
 					gchar *val;
 
-					val = ews_oab_read_upto (G_INPUT_STREAM (priv->fis), '\0', cancellable, error);
-					if (!val) {
+					if (prop_type == EWS_PTYP_MULTIPLEBINARY) {
+						val = ews_decode_binary (eod, cancellable, error);
+					} else {
+						val = ews_oab_read_upto (G_INPUT_STREAM (priv->fis), '\0', cancellable, error);
+					}
+
+					if (*error) {
 						g_slist_foreach (list, (GFunc) g_free, NULL);
 						g_slist_free (list);
 						return NULL;
 					}
-
-					d (g_print ("prop id %X prop type: multi-str '%s'\n", prop_id, val));
-					list = g_slist_prepend (list, val);
 				}
 
 			}
-			ret_val = list;
 
 			break;
 		}
@@ -762,7 +732,9 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 }
 
 static void
-ews_destroy_oab_prop (guint32 prop_id, gpointer val)
+ews_destroy_oab_prop (guint32 prop_id,
+                      gpointer val,
+                      gboolean delete_files)
 {
 	guint32 prop_type;
 
@@ -772,17 +744,14 @@ ews_destroy_oab_prop (guint32 prop_id, gpointer val)
 		case EWS_PTYP_INTEGER32:
 		case EWS_PTYP_BOOLEAN:
 			break;
-		case EWS_PTYP_BINARY:
-			g_bytes_unref(val);
-			break;
 		case EWS_PTYP_STRING8:
 		case EWS_PTYP_STRING:
+		case EWS_PTYP_BINARY:
 			g_free ((gchar *) val);
 			break;
 		case EWS_PTYP_MULTIPLEBINARY:
-			g_slist_foreach ((GSList *) val, (GFunc) g_bytes_unref, NULL);
-			g_slist_free ((GSList *) val);
-			break;
+			if (delete_files)
+				g_slist_foreach ((GSList *) val, (GFunc) g_unlink, NULL);
 		case EWS_PTYP_MULTIPLESTRING8:
 		case EWS_PTYP_MULTIPLESTRING:
 			g_slist_foreach ((GSList *) val, (GFunc) g_free, NULL);
@@ -795,6 +764,82 @@ ews_destroy_oab_prop (guint32 prop_id, gpointer val)
 	}
 }
 
+static const gchar *
+ews_decode_addressbook_get_display_type (guint32 value)
+{
+	const gchar *display_type = NULL;
+
+	switch (value) {
+		case EWS_DT_MAILUSER:
+		/*
+		 * DT_MAILUSER means that the display type could be, besides itself, one of
+		 * these extended types: DT_ROOM, DT_EQUIPMENT, DT_SEC_DISTLIST. Considering
+		 * this, we can just ignore this case here and treat it properly in the
+		 * ews_decode_addressbook_get_display_type_extended() function.
+		 */
+			break;
+		case EWS_DT_DISTLIST:
+			display_type = "DT_DISTLIST";
+			break;
+		case EWS_DT_FORUM:
+			display_type = "DT_FORUM";
+			break;
+		case EWS_DT_AGENT:
+			display_type = "DT_AGENT";
+			break;
+		case EWS_DT_ORGANIZATION:
+			display_type = "DT_ORGANIZATION";
+			break;
+		case EWS_DT_PRIVATE_DISTLIST:
+			display_type = "DT_PRIVATE_DISTLIST";
+			break;
+		case EWS_DT_REMOTE_MAILUSER:
+			display_type = "DT_PRIVATE_MAILUSER";
+			break;
+	}
+
+	return display_type;
+}
+
+static const gchar *
+ews_decode_addressbook_get_display_type_extended (guint32 value)
+{
+	const gchar *display_type = "DT_MAILUSER";
+
+	switch (value) {
+		case EWS_DT_ROOM:
+			display_type = "DT_ROOM";
+			break;
+		case EWS_DT_EQUIPMENT:
+			display_type = "DT_EQUIPMENT";
+			break;
+		case EWS_DT_SEC_DISTLIST:
+			display_type = "DT_SEC_DISTLIST";
+			break;
+	}
+
+	return display_type;
+}
+
+static void
+ews_decode_addressbook_write_display_type (EContact **contact,
+					   guint32 value,
+					   gboolean extended)
+{
+	EVCardAttribute *attr;
+	const gchar *display_type;
+
+	if (extended)
+		display_type = ews_decode_addressbook_get_display_type_extended (value);
+	else
+		display_type = ews_decode_addressbook_get_display_type (value);
+
+	if (display_type != NULL) {
+		attr = e_vcard_attribute_new (NULL, "X-EWS-KIND");
+		e_vcard_add_attribute_with_value (E_VCARD (*contact), attr, display_type);
+	}
+}
+
 /**
  * ews_decode_addressbook_record 
  * @eod: 
@@ -843,6 +888,12 @@ ews_decode_addressbook_record (EwsOabDecoder *eod,
 
 		val = ews_decode_oab_prop (eod, prop_id, cancellable, error);
 
+		if (prop_id == EWS_PT_DISPLAY_TYPE)
+			ews_decode_addressbook_write_display_type (&contact, GPOINTER_TO_UINT (val), FALSE);
+
+		if (prop_id == EWS_PT_DISPLAY_TYPE_EX)
+			ews_decode_addressbook_write_display_type (&contact, GPOINTER_TO_UINT (val), TRUE);
+
 		/* Check the contact map and store the data in EContact */
 		index = g_hash_table_lookup (priv->prop_index_dict, GINT_TO_POINTER (prop_id));
 		if (contact && index) {
@@ -853,7 +904,10 @@ ews_decode_addressbook_record (EwsOabDecoder *eod,
 			else
 				prop_map[i - 1].defered_populate_function (dset, prop_id, val);
 		}
-		ews_destroy_oab_prop (prop_id, val);
+
+		/* delete the binary file if we do not have the property in the index or if there
+		 * was an error */
+		ews_destroy_oab_prop (prop_id, val, (*error || !index) ? TRUE : FALSE);
 		if (*error)
 			goto exit;
 	}
@@ -1093,3 +1147,78 @@ ews_oab_decoder_get_contact_from_offset (EwsOabDecoder *eod,
 	return contact;
 }
 
+#ifdef TEST_BUILD
+
+struct _db_data {
+	GSList *contact_collector;
+	guint collected_length;
+	EBookBackendSqliteDB *summary;
+	const gchar *folderid;
+};
+
+static void
+ews_test_store_contact (EContact *contact,
+                        goffset offset,
+                        guint percent,
+                        gpointer user_data,
+                        GError **error)
+{
+	struct _db_data *data = (struct _db_data *) user_data;
+
+	data->contact_collector = g_slist_prepend (data->contact_collector, g_object_ref (contact));
+	data->collected_length += 1;
+
+	if (data->collected_length == 1000 || percent >= 100) {
+		data->contact_collector = g_slist_reverse (data->contact_collector);
+		e_book_backend_sqlitedb_add_contacts (data->summary, data->folderid, data->contact_collector, FALSE, error);
+		g_print ("percent complete %d \n", percent);
+
+		g_slist_foreach (data->contact_collector, (GFunc) g_object_unref, NULL);
+		g_slist_free (data->contact_collector);
+		data->contact_collector = NULL;
+		data->collected_length = 0;
+	}
+}
+
+gint
+main (gint argc,
+      gchar *argv[])
+{
+	EBookBackendSqliteDB *summary;
+	EwsOabDecoder *eod;
+	GError *err = NULL;
+	GTimer *timer;
+	struct _db_data data;
+
+	g_type_init ();
+
+	if (argc != 3) {
+		g_print ("Pass the oab filename  and cache dir as argument \n");
+		return -1;
+	}
+
+	summary = e_book_backend_sqlitedb_new (argv[2], "dum", "de", "dum", TRUE, NULL);
+	eod = ews_oab_decoder_new (argv[1], argv[2], &err);
+
+	data.contact_collector = NULL;
+	data.collected_length = 0;
+	data.summary = summary;
+	data.folderid = "de";
+
+	timer = g_timer_new ();
+	g_timer_start (timer);
+	if (!ews_oab_decoder_decode (eod, ews_test_store_contact, &data, NULL, &err)) {
+		g_print ("Unable to decode %s \n", err->message);
+	}
+	g_timer_stop (timer);
+	g_print ("Time elapsed %lf \n", g_timer_elapsed (timer, NULL));
+
+	if (err)
+		g_clear_error (&err);
+
+	g_object_unref (eod);
+	g_object_unref (summary);
+
+	return 0;
+}
+#endif /* TEST_BUILD */
